package example.energy.coolemall;

import schedframe.Parameter;
import schedframe.events.scheduling.EventReason;
import schedframe.resources.computing.ComputingResource;
import schedframe.resources.computing.Processor;
import schedframe.resources.computing.profiles.energy.EnergyEvent;
import schedframe.resources.computing.recs.Node;
import schedframe.scheduling.manager.tasks.JobRegistry;
import example.energy.BaseEnergyEstimationPlugin;

public class NodeEnergyEstimationPlugin extends BaseEnergyEstimationPlugin {

	private static int OUT_START_ID = 0;
	private static int OUT_END_ID = 8;
	
	public double estimatePowerConsumption(EnergyEvent event, JobRegistry jobRegistry,
			ComputingResource resource) {
		double powerConsumption = 0;
		Node node = (Node) resource;
		for(Processor cpu: node.getProcessors()){
			try{
				powerConsumption = powerConsumption + cpu.getPowerInterface().getRecentPowerUsage().getValue();
			} catch (Exception e){
				
			}
		}
		try {
			powerConsumption = powerConsumption + node.getPowerInterface().getPowerConsumption(node.getPowerInterface().getPowerState());
		} catch (NoSuchFieldException e) {
			Parameter param = node.getResourceCharacteristic().getParameters().get("maxPower");
			if(param != null)
				powerConsumption = powerConsumption + Double.valueOf(param.get(0).getContent());
		}

		return powerConsumption;
	}

	public double estimateAirThroughput(EnergyEvent event, JobRegistry jobRegistry, ComputingResource resource) {

		double airThroughput = 0;
		try {
			if(event.getReason() == EventReason.SIM_INIT)
				return airThroughput;
				//airThroughput = resource.getAirThroughputInterface().getAirFlow(StandardAirThroughputStateName.FAN_OFF);
			else
				airThroughput = resource.getAirThroughputInterface().getAirFlow(resource.getAirThroughputInterface().getAirThroughputState());
		} catch (NoSuchFieldException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		Integer resId = Integer.parseInt(resource.getName().split("_")[1]);
		//we estimate the air flow related to the outlets (nodes placed in the outlet row of RECS)
		if(resId >= OUT_START_ID  && resId <= OUT_END_ID){
			for(ComputingResource compResource: resource.getParent().getChildren()){
				Integer id = Integer.parseInt(compResource.getName().split("_")[1]);
				//we take into account the inlet air flow (nodes placed "behind" the given node)
				if(id - resId == 9 || id - resId == -9){
					try {
						airThroughput = airThroughput + compResource.getAirThroughputInterface().getAirFlow(compResource.getAirThroughputInterface().getAirThroughputState())/2;
					} catch (NoSuchFieldException e) {
		
					}
				}
			}
		}

		return airThroughput;
	}

	public double estimateTemperature(EnergyEvent event, JobRegistry jobRegistry, ComputingResource resource) {
		double tout = 0;
		double tin = 21.3;
		double Q = 0.0056085;
		double C = 1004;
		double delta1 = 1.780594389/2;
		double delta2 = 2.162043729/2;
		double ro = 1.168;
		double power1 = 0;
		double power2 = 0;
		
		Integer resId = Integer.parseInt(resource.getName().split("_")[1]);
		
		//we estimate outlet temperature (related to the temperature of the nodes placed in outlet row)
		for(ComputingResource compResource: resource.getParent().getChildren()){
			Integer id = Integer.parseInt(compResource.getName().split("_")[1]);
			//we take into account the power of nodes placed "behind" the given nodes
			if(id - resId == 9){
				power1 = compResource.getPowerInterface().getRecentPowerUsage().getValue();
				power2 = resource.getPowerInterface().getRecentPowerUsage().getValue();
			} else if(id - resId == -9){
				power1 = resource.getPowerInterface().getRecentPowerUsage().getValue();
				power2 = compResource.getPowerInterface().getRecentPowerUsage().getValue();
			}
		}
		
		tout = tin + delta1 * (power1/(Q * C * ro)) +  delta2 * (power2/(Q * C * ro));;
		return tout;
	}
	
}
